unit ParamsEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DataTypes, StdCtrls, ExtCtrls;

type
  TFormParamsEdit = class(TForm)
    BtnOk: TButton;
    BtnCancel: TButton;
    Panel: TPanel;
    Label1: TLabel;
    EditDiameter: TEdit;
    EditNSamplesScrSide: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    EditDistance: TEdit;
    EditWaveLength: TEdit;
    Label4: TLabel;
    Label8: TLabel;
    EditDelta: TEdit;
    EditBetta: TEdit;
    EditPointS: TEdit;
    EditPointDeltaX: TEdit;
    EditScreenLength: TEdit;
    EditNSamplesScrLen: TEdit;
    RGCalcMode: TRadioGroup;
    RBNormal: TRadioButton;
    RBParallel: TRadioButton;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label5: TLabel;
    EditScreenSide: TEdit;
    Label6: TLabel;
    EditPointDeltaY: TEdit;
    Label7: TLabel;
    EditActWaveLength: TEdit;
    Label9: TLabel;
    EditCirclePoints: TEdit;
    EditRadiusDispersion: TEdit;
    Label15: TLabel;
    Label16: TLabel;
    procedure FormDestroy(Sender: TObject);
    procedure EditExit(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure BtnOkClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    TheParams : TParams;
    IncorrectData : boolean;

    procedure LoadParams;
    procedure SaveParams;
    procedure LoadSecondaryParams;
    procedure ProcessField(AField : TEdit);
  public
    function Execute(Params : TParams) : boolean;
  end;

var
  FormParamsEdit: TFormParamsEdit;

implementation

const
  FIELD_INCORRECT = $80000000;

{$R *.dfm}

// ========   ========
function TFormParamsEdit.Execute(Params : TParams) : boolean;
begin
  //    
  if TheParams = nil then TheParams := TParams.Create;
  TheParams.Assign(Params);

  //  
  LoadParams;
  ActiveControl := BtnOk;

  //  .  ,  .    
  if ShowModal = mrOk then begin
    Result := Params.Differs(TheParams);
    Params.Assign(TheParams);
  end else
    Result := False;
end;

// ========   ========
// -    
procedure TFormParamsEdit.LoadParams;
begin
  //   (     )
  EditWaveLength.Tag := 0;
  EditActWaveLength.Tag := 0;
  EditDistance.Tag := 0;
  EditScreenLength.Tag := 0;
  EditScreenSide.Tag := 0;
  EditDelta.Tag := 0;
  EditBetta.Tag := 0;
  EditDiameter.Tag := 0;
  EditPointS.Tag := 0;
  EditPointDeltaX.Tag := 0;
  EditPointDeltaY.Tag := 0;
  EditCirclePoints.Tag := 0;
  EditRadiusDispersion.Tag := 0;

  EditWaveLength.Font.Color := clWindowText;
  EditActWaveLength.Font.Color := clWindowText;
  EditDistance.Font.Color := clWindowText;
  EditScreenLength.Font.Color := clWindowText;
  EditScreenSide.Font.Color := clWindowText;
  EditDelta.Font.Color := clWindowText;
  EditBetta.Font.Color := clWindowText;
  EditDiameter.Font.Color := clWindowText;
  EditPointS.Font.Color := clWindowText;
  EditPointDeltaX.Font.Color := clWindowText;
  EditPointDeltaY.Font.Color := clWindowText;
  EditCirclePoints.Font.Color := clWindowText;
  EditRadiusDispersion.Font.Color := clWindowText;

  //     (   Params)
  // -  
  with TheParams do begin
    EditWaveLength.Text := Format('%.2f',[WaveLength]);
    EditActWaveLength.Text := Format('%.2f',[ActWaveLength]);
    EditDistance.Text := Format('%.1f',[Distance]);
    EditScreenLength.Text := Format('%.1f',[ScreenLength]);
    EditScreenSide.Text := Format('%.1f',[ScreenSide]);
    EditDelta.Text := Format('%.2f',[Delta]);
    EditBetta.Text := Format('%.3f',[Betta]);
    EditDiameter.Text := Format('%.1f',[Diameter]);
    EditPointS.Text := Format('%.1f',[PointS]);
    EditPointDeltaX.Text := Format('%.1f',[PointDeltaX]);
    EditPointDeltaY.Text := Format('%.1f',[PointDeltaY]);
    EditCirclePoints.Text := IntToStr(CirclePoints);
    EditRadiusDispersion.Text := IntToStr(RadiusDispersion);
  end;
  // -  
  RBNormal.Checked := TheParams.CalcMode = cmNormal;

  LoadSecondaryParams;
end;

// -       ( ,     )
procedure TFormParamsEdit.SaveParams;
begin
  // -  
  if RBNormal.Checked then TheParams.CalcMode := cmNormal
  else TheParams.CalcMode := cmParallel;
end;

procedure TFormParamsEdit.LoadSecondaryParams;
begin
  with TheParams do begin
    EditNSamplesScrSide.Text := IntToStr(NSamplesScrSide);
    EditNSamplesScrLen.Text := IntToStr(NSamplesScrLen);
  end;
end;

// -   
procedure TFormParamsEdit.ProcessField(AField : TEdit);
var
  c1 : integer;
  S : string;
  Val : single;
  IncorrectPresent : boolean;
begin
  //    (      )
  S := AField.Text;
  for c1 := 1 to Length(AField.Text) do
    if S[c1] in ['.',','] then S[c1] := DECIMALSEPARATOR;

  try //  
    Val := StrToFloat(S);
    AField.Text := S;
    //    
    AField.Tag := AField.Tag and (not FIELD_INCORRECT);
    //   

    with TheParams do
      if AField = EditWaveLength then WaveLength := val
      else if AField = EditActWaveLength then ActWaveLength := val
      else if AField = EditDistance then Distance := val
      else if AField = EditScreenLength then ScreenLength := val
      else if AField = EditScreenSide then ScreenSide := val
      else if AField = EditDelta then Delta := val
      else if AField = EditBetta then Betta := val
      else if AField = EditDiameter then Diameter := val
      else if AField = EditPointS then PointS := val
      else if AField = EditPointDeltaX then PointDeltaX := val
      else if AField = EditPointDeltaY then PointDeltaY := val
      else if AField = EditCirclePoints then CirclePoints := Trunc(val)
      else if AField = EditRadiusDispersion then RadiusDispersion := Trunc(val);

    //    
    IncorrectPresent := EditWaveLength.Tag and FIELD_INCORRECT <> 0;
    IncorrectPresent := IncorrectPresent or (EditActWaveLength.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditDistance.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditScreenLength.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditScreenSide.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditDelta.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditBetta.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditDiameter.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditPointS.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditPointDeltaX.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditPointDeltaY.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditCirclePoints.Tag and FIELD_INCORRECT <> 0);
    IncorrectPresent := IncorrectPresent or (EditRadiusDispersion.Tag and FIELD_INCORRECT <> 0);

    if not IncorrectPresent then begin
      LoadSecondaryParams;
    end;
  except
    AField.Tag := dword(AField.Tag) or FIELD_INCORRECT;    //    
    AField.Font.Color := clRed;                     //  
    IncorrectPresent := true;
  end;
  IncorrectData := IncorrectPresent;
end;


// ========   ========
// - 
procedure TFormParamsEdit.FormDestroy(Sender: TObject);
begin
  TheParams.Free;
end;

procedure TFormParamsEdit.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = chr($0d)) and (ActiveControl is TEdit) then begin
    ProcessField(ActiveControl as TEdit);
    (ActiveControl as TEdit).SelectAll;
  end;
end;

// -  
procedure TFormParamsEdit.EditExit(Sender: TObject);
begin
  ProcessField(Sender as TEdit);
end;

procedure TFormParamsEdit.EditChange(Sender: TObject);
begin
  (Sender as TEdit).Font.Color := clWindowText;
end;

procedure TFormParamsEdit.BtnOkClick(Sender: TObject);
begin
  if ActiveControl.ClassType = TEdit then ProcessField(ActiveControl as TEdit);
  if IncorrectData then MessageDlg('  !',mtError,[mbOK],0)
  else begin
    SaveParams;
    ModalResult := mrOk;
  end;
end;

end.
